import { Component, EventEmitter, inject, input, Input, OnInit, Output } from '@angular/core';
import { FormControl, ReactiveFormsModule, Validators } from '@angular/forms';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { ActivatedRoute } from '@angular/router';
import RedeemableAction from '../../shared/models/redeemable-action';

@Component({
  selector: 'action-dropdown',
  imports: [MatAutocompleteModule, MatFormFieldModule, MatInputModule, ReactiveFormsModule],
  templateUrl: './action-dropdown.component.html',
  styleUrl: './action-dropdown.component.scss'
})
export class ActionDropdownComponent implements OnInit {
  @Input() formControl = new FormControl<RedeemableAction | string | undefined>(undefined);
  @Input() errorMessages: { [errorKey: string]: string } = {}

  @Input() search: boolean = false;
  @Input() actions: RedeemableAction[] = [];
  @Input() action: string | undefined;
  @Output() readonly actionChange = new EventEmitter<string>();

  private readonly route = inject(ActivatedRoute);

  errorMessageKeys: string[] = []


  constructor() {
    this.route.data.subscribe(data => {
      if (!data['redeemableActions'])
        return;

      this.actions = data['redeemableActions'];
    });
  }

  ngOnInit(): void {
    this.errorMessageKeys = Object.keys(this.errorMessages);

    if (!this.action)
      return;

    const action = this.actions.find(r => r.name == this.action);
    this.formControl.setValue(action);
  }

  get filteredActions() {
    const value = this.formControl.value;
    if (typeof value == 'string') {
      return this.actions.filter(r => r.name.toLowerCase().includes(value.toLowerCase()));
    }
    return this.actions;
  }

  select(event: RedeemableAction) {
    this.actionChange.emit(event.name);
  }

  input() {
    if (this.search && typeof this.formControl.value == 'string') {
      this.actionChange.emit(this.formControl.value);
    }
  }

  blur() {
    if (!this.search && typeof this.formControl.value == 'string') {
      const name = this.formControl.value;
      const nameLower = name.toLowerCase();
      let newValue: RedeemableAction | undefined = undefined;
      const insenstiveActions = this.filteredActions.filter(a => a.name.toLowerCase() == nameLower);
      if (insenstiveActions.length > 1) {
        const sensitiveAction = insenstiveActions.find(a => a.name == name);
        newValue = sensitiveAction ?? undefined;
      } else if (insenstiveActions.length == 1) {
        newValue = insenstiveActions[0];
      }

      if (newValue && newValue.name != this.formControl.value) {
        this.formControl.setValue(newValue);
        this.actionChange.emit(newValue.name);
      } else if (!newValue)
        this.actionChange.emit(undefined);
    }
  }

  displayFn(value: any) {
    return value?.name;
  }
}
