import { Component, inject, Inject, OnInit, PLATFORM_ID } from '@angular/core';
import { ApiAuthenticationService } from '../../shared/services/api/api-authentication.service';
import { MatCardModule } from '@angular/material/card';
import { MatSelectModule } from '@angular/material/select';
import { HttpClient } from '@angular/common/http';
import { isPlatformBrowser } from '@angular/common';
import { environment } from '../../../environments/environment';
import EventService from '../../shared/services/EventService';
import { HermesClientService } from '../../hermes-client.service';
import { Router } from '@angular/router';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { User } from '../../shared/models/user';
import { UserService } from '../../shared/services/user.service';

@Component({
  selector: 'impersonation',
  standalone: true,
  imports: [
    MatCardModule,
    MatSelectModule,
    ReactiveFormsModule,
  ],
  templateUrl: './impersonation.component.html',
  styleUrl: './impersonation.component.scss'
})
export class ImpersonationComponent implements OnInit {
  private readonly events = inject(EventService);
  private readonly userService = inject(UserService);

  impersonationControl = new FormControl<string | undefined>(undefined);
  users: User[];

  constructor(private client: HermesClientService, private auth: ApiAuthenticationService, private router: Router, private http: HttpClient, @Inject(PLATFORM_ID) private platformId: Object) {
    this.users = [];
  }

  ngOnInit(): void {
    if (!isPlatformBrowser(this.platformId)) {
      return;
    }

    this.userService.fetch().subscribe(users => {
      this.users = users.filter((d: any) => d.name != this.auth.getUsername());
      const id = this.auth.getImpersonatedId();
      if (id && this.users.find(u => u.id == id)) {
        this.impersonationControl.setValue(id);
      }
    });

    this.impersonationControl.valueChanges.subscribe((impersonationId) => {
      if (!this.auth.isAdmin() || impersonationId == this.auth.getImpersonatedId())
        return;

      if (!impersonationId) {
        this.http.delete(environment.API_HOST + '/admin/impersonate', {
          headers: {
            'Authorization': 'Bearer ' + localStorage.getItem('jwt')
          },
          body: {
            impersonation: impersonationId
          }
        }).subscribe(async (data: any) => {
          this.impersonationControl.setValue(undefined);
          this.client.disconnect(true);
          this.events.emit('impersonation', undefined);
        });
      } else {
        this.http.put(environment.API_HOST + '/admin/impersonate', {
          impersonation: impersonationId
        }, {
          headers: {
            'Authorization': 'Bearer ' + localStorage.getItem('jwt')
          }
        }).subscribe(async (data: any) => {
          this.impersonationControl.setValue(impersonationId);
          this.client.disconnect(true);
          this.events.emit('impersonation', impersonationId);
          await this.router.navigate(['tts-login']);
        });
      }
    });
  }

  public isAdmin() {
    return this.auth.isAdmin();
  }

  public getUsername() {
    return this.auth.getUsername();
  }
}