import { Component, inject, OnDestroy } from '@angular/core';
import { Connection } from '../../shared/models/connection';
import { ActivatedRoute } from '@angular/router';
import { ConnectionListComponent } from "../connection-list/connection-list.component";
import { Subscription } from 'rxjs';
import { ConnectionService } from '../../shared/services/connection.service';

@Component({
  selector: 'connections',
  imports: [ConnectionListComponent],
  templateUrl: './connections.component.html',
  styleUrl: './connections.component.scss'
})
export class ConnectionsComponent implements OnDestroy {
  private readonly route = inject(ActivatedRoute);
  private readonly connectionService = inject(ConnectionService);
  subscriptions: (Subscription | undefined)[] = [];
  connections: Connection[] = [];


  constructor() {
    this.route.data.subscribe(payload => {
      this.connections = payload['connections'] ?? [];
    });

    this.subscriptions.push(this.connectionService.delete$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this.connectionService.fetch().subscribe(connections => this.connections = connections);
    }));
  }

  ngOnDestroy(): void {
    for (let subscription of this.subscriptions) {
      if (subscription)
        subscription.unsubscribe();
    }
  }
}
