import { HttpClient } from '@angular/common/http';
import { Component, inject } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from '@angular/forms';
import { MatButtonModule } from '@angular/material/button';
import { MatCardModule } from '@angular/material/card';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { MatSelectModule } from '@angular/material/select';
import EventService from '../../shared/services/EventService';

@Component({
  selector: 'key-item-edit',
  imports: [
    MatButtonModule,
    MatCardModule,
    MatIconModule,
    MatFormFieldModule,
    MatInputModule,
    MatSelectModule,
    ReactiveFormsModule,
  ],
  templateUrl: './key-item-edit.component.html',
  styleUrl: './key-item-edit.component.scss'
})
export class KeyItemEditComponent {
  private readonly http = inject(HttpClient);
  private readonly events = inject(EventService);
  readonly data = inject<{ name: string }>(MAT_DIALOG_DATA);

  readonly labelControl = new FormControl<string>('', [Validators.required]);
  readonly form = new FormGroup({
    name: this.labelControl,
  });

  readonly dialogRef = inject(MatDialogRef<KeyItemEditComponent>);

  responseError: string | undefined;
  waitForResponse = false;


  ngOnInit(): void {
    this.labelControl.setValue(this.data.name);
  }

  submit(): void {
    if (this.form.invalid || this.waitForResponse) {
      return;
    }

    const label = this.labelControl.value;
    this.http.post('/api/keys', { label },
      {
        headers: {
          'Authorization': 'Bearer ' + localStorage.getItem('jwt')
        }
      }).subscribe(async (d: any) => {
        this.events.emit('add_api_key', {
          id: d.key,
          label: d.label,
        });

        this.dialogRef.close();
      });
  }
}
