import { Component, inject, input } from '@angular/core';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatFormFieldModule } from '@angular/material/form-field';
import { ReactiveFormsModule } from '@angular/forms';
import { HttpClient } from '@angular/common/http';
import ApiKey from '../../shared/models/api-key';
import EventService from '../../shared/services/EventService';

@Component({
  selector: 'key-item',
  imports: [
    MatButtonModule,
    MatIconModule,
    MatFormFieldModule,
    ReactiveFormsModule,
  ],
  templateUrl: './key-item.component.html',
  styleUrl: './key-item.component.scss'
})
export class KeyItemComponent {
  private readonly http = inject(HttpClient);
  private readonly events = inject(EventService);

  key = input.required<ApiKey>();
  isVisible: boolean = false;
  waitForResponse = false;

  delete() {
    if (this.waitForResponse) {
      return;
    }

    const key_id = this.key().id;
    this.http.delete('/api/keys',
      {
        body: {
          key: key_id,
        },
        headers: {
          'Authorization': 'Bearer ' + localStorage.getItem('jwt')
        }
      }).subscribe(async (d: any) => {
        this.events.emit('delete_api_key', key_id);
        this.waitForResponse = false;
      });
  }
}
