import { AsyncPipe } from '@angular/common';
import { Component, Input } from '@angular/core';
import { FormControl, FormsModule, ReactiveFormsModule, Validators } from '@angular/forms'
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { MatButtonModule } from '@angular/material/button';
import { MatInputModule } from '@angular/material/input';
import { map, Observable, startWith } from 'rxjs';

const Policies = [
  { path: "tts", description: "Anything to do with TTS" },
  { path: "tts.chat", description: "Anything to do with chat" },
  { path: "tts.chat.bits.read", description: "To read chat messages with bits via TTS" },
  { path: "tts.chat.messages.read", description: "To read chat messages via TTS" },
  { path: "tts.chat.redemptions.read", description: "To read channel point redemption messages via TTS" },
  { path: "tts.chat.subscriptions.read", description: "To read chat messages from subscriptions via TTS" },
  { path: "tts.commands", description: "To execute commands for TTS" },
  { path: "tts.commands.nightbot", description: "To use !nightbot command" },
  { path: "tts.commands.obs", description: "To use !obs command" },
  { path: "tts.commands.refresh", description: "To use !refresh command" },
  { path: "tts.commands.skip", description: "To use !skip command" },
  { path: "tts.commands.skipall", description: "To use !skipall command" },
  { path: "tts.commands.tts", description: "To use !tts command" },
  { path: "tts.commands.tts.join", description: "To use !tts join command" },
  { path: "tts.commands.tts.leave", description: "To use !tts leave command" },
  { path: "tts.commands.version", description: "To use !version command" },
  { path: "tts.commands.voice", description: "To use !voice command" },
  { path: "tts.commands.voice.admin", description: "To use !voice command on others" },
];

@Component({
  selector: 'policy-dropdown',
  imports: [
    AsyncPipe,
    FormsModule,
    MatAutocompleteModule,
    MatButtonModule,
    MatInputModule,
    ReactiveFormsModule,
  ],
  templateUrl: './policy-dropdown.component.html',
  styleUrl: './policy-dropdown.component.scss'
})
export class PolicyDropdownComponent {
  @Input() policy: string | null = '';
  @Input({ alias: 'control' }) policyControl = new FormControl('', [Validators.required]);
  filteredPolicies: Observable<string[]>;

  constructor() {
    this.filteredPolicies = this.policyControl.valueChanges.pipe(
      startWith(''),
      map(value => this._filter(value || '')),
    );
  }

  ngOnInit() {
    this.policyControl.setValue(this.policy);
    this.filteredPolicies = this.policyControl.valueChanges.pipe(
      startWith(''),
      map(value => this._filter(value || '')),
    );
  }

  private _filter(value: string): string[] {
    const filterValue = value.toLowerCase();
    const names = Policies.map(p => p.path);
    if (names.includes(filterValue)) {
      return names;
    }

    return names.filter(option => option.toLowerCase().includes(filterValue));
  }
}
