import { inject, Injectable } from '@angular/core';
import { HermesClientService } from '../../hermes-client.service';
import { map, Observable, of } from 'rxjs';
import RedeemableAction from '../models/redeemable-action';
import EventService from './EventService';

@Injectable({
  providedIn: 'root'
})
export default class RedeemableActionService {
  private readonly client = inject(HermesClientService);
  private readonly events = inject(EventService);
  private data: RedeemableAction[] = []
  private loaded = false;
  create$: Observable<any> | undefined;
  update$: Observable<any> | undefined;
  delete$: Observable<any> | undefined;


  constructor() {
    this.create$ = this.client.filterByRequestType('create_redeemable_action');
    this.update$ = this.client.filterByRequestType('update_redeemable_action');
    this.delete$ = this.client.filterByRequestType('delete_redeemable_action');

    this.create$?.subscribe(d => this.data.push(d.data));
    this.update$?.subscribe(d => {
      const action = this.data.find(r => r.name == d.data.name);
      if (action) {
        action.type = d.data.type;
        action.data = d.data.data;
      }
    });
    this.delete$?.subscribe(d => this.data = this.data.filter(r => r.name != d.request.data.name));

    this.events.listen('tts_logoff', () => {
      this.data = [];
      this.loaded = false;
    });
  }

  fetch() {
    if (this.loaded) {
      return of(this.data);
    }

    const $ = this.client.filterByRequestType('get_redeemable_actions')!.pipe(map(d => d.data));
    $.subscribe(d => {
      this.data = d;
      this.loaded = true;
    });
    this.client.fetchRedeemableActions();
    return $;
  }
}
