import { Component, inject } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA, MatDialogModule } from '@angular/material/dialog';
import { Filter } from '../../shared/models/filter';
import { FormControl, FormGroup, FormsModule, ReactiveFormsModule, Validators } from '@angular/forms';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { MatButtonModule } from '@angular/material/button';
import { MatCardModule } from '@angular/material/card';
import { MatSelectChange, MatSelectModule } from '@angular/material/select';
import { HermesClientService } from '../../hermes-client.service';
import { MatIconModule } from '@angular/material/icon';

@Component({
  selector: 'tts-filter-item-edit',
  standalone: true,
  imports: [
    FormsModule,
    MatButtonModule,
    MatCardModule,
    MatDialogModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    MatSelectModule,
    ReactiveFormsModule,
  ],
  templateUrl: './filter-item-edit.component.html',
  styleUrl: './filter-item-edit.component.scss'
})
export class FilterItemEditComponent {
  private readonly client = inject(HermesClientService);

  private readonly dialogRef = inject(MatDialogRef<FilterItemEditComponent>);
  readonly data = inject<Filter>(MAT_DIALOG_DATA);

  readonly regexOptions = [
    {
      name: 'Ignore Case',
      flag: 1
    },
    {
      name: 'Explicit Capture',
      flag: 4
    },
    {
      name: 'Ignore Pattern Whitespace',
      flag: 16
    },
    {
      name: 'No Backtracking',
      flag: 1024
    },
  ];

  readonly searchControl = new FormControl(this.data.search, [Validators.required]);
  readonly replaceControl = new FormControl(this.data.replace);
  readonly flagControl = new FormControl<string[]>(this.optionsSelected);
  readonly forms = new FormGroup({
    search: this.searchControl,
    replace: this.replaceControl,
    flag: this.flagControl,
  });

  waitForResponse: boolean = false;
  responseError: string | undefined;


  get optionsSelected() {
    const flag = this.data.flag;
    return this.regexOptions.filter(o => (flag & o.flag) > 0).map(o => o.name);
  }

  onSaveClick(): void {
    if (this.forms.invalid || this.waitForResponse)
      return;

    this.waitForResponse = true;
    this.responseError = undefined;

    const search = this.searchControl.value!;
    const replace = this.replaceControl.value!;
    const flag = this.data.flag;

    if (!this.data.id) {
      this.client.first(d => d.op == 4 && d.d.request.type == 'create_tts_filter' && d.d.data.search == search && d.d.data.replace == replace && d.d.data.flag == flag)
        ?.subscribe({
          next: d => {
            if (d.error)
              this.responseError = d.error;
            else
              this.dialogRef.close(d.d.data);
          },
          error: () => this.responseError = 'Something went wrong.',
          complete: () => this.waitForResponse = false,
        });
      this.client.createTTSFilter(search, replace, flag);
    } else {
      this.client.first(d => d.op == 4 && d.d.request.type == 'update_tts_filter' && d.d.data.id == this.data.id && d.d.data.search == search && d.d.data.replace == replace && d.d.data.flag == flag)
        ?.subscribe({
          next: d => {
            if (d.error)
              this.responseError = d.error;
            else
              this.dialogRef.close(d.d.data);
          },
          error: () => this.responseError = 'Something went wrong.',
          complete: () => this.waitForResponse = false,
        });
      this.client.updateTTSFilter(this.data.id, search, replace, flag);
    }
  }

  onCancelClick(): void {
    this.dialogRef.close();
  }

  onSelectionChange(event: MatSelectChange) {
    this.data.flag = event.value.reduce((acc: number, option: any) => acc | option.flag, 0);
  }

  compare(o: any, s: any) {
    return o.flag == s || o.name == s;
  }
}
