import { HttpClient } from '@angular/common/http';
import { Component, inject, OnInit } from '@angular/core';
import { FormControl, ReactiveFormsModule, Validators } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { ActionItemEditComponent } from '../../actions/action-item-edit/action-item-edit.component';
import { HermesClientService } from '../../hermes-client.service';
import { MatCardModule } from '@angular/material/card';
import { MatButtonModule } from '@angular/material/button';
import { Group } from '../../shared/models/group';
import { MatInputModule } from '@angular/material/input';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';

@Component({
  selector: 'app-twitch-user-item-add',
  imports: [
    MatButtonModule,
    MatCardModule,
    MatIconModule,
    MatFormFieldModule,
    MatInputModule,
    ReactiveFormsModule,
  ],
  templateUrl: './twitch-user-item-add.component.html',
  styleUrl: './twitch-user-item-add.component.scss'
})
export class TwitchUserItemAddComponent implements OnInit {
  private readonly client = inject(HermesClientService);
  private readonly http = inject(HttpClient);

  readonly data = inject<{ username: string, group: Group }>(MAT_DIALOG_DATA);
  readonly usernameControl = new FormControl('', [Validators.required]);
  readonly dialogRef = inject(MatDialogRef<ActionItemEditComponent>);

  responseError: string | undefined;
  waitForResponse = false;


  ngOnInit(): void {
    this.usernameControl.setValue(this.data.username);
  }

  submit() {
    if (this.usernameControl.invalid || this.waitForResponse || !this.client.api_key) {
      return;
    }

    this.waitForResponse = true;
    this.responseError = undefined;

    const username = this.usernameControl.value!.toLowerCase();
    this.http.get('/api/auth/twitch/users?login=' + username, {
      headers: {
        'x-api-key': this.client.api_key,
      }
    })
      .subscribe((response: any) => {
        if (!response.user) {
          this.waitForResponse = false;
          this.responseError = 'Twitch username does not exist.';
          return;
        }

        this.client.first((d: any) => d.op == 4 && d.d.request.type == 'create_group_chatter' && d.d.request.data.chatter == response.user.id)
          .subscribe({
            next: (d) => {
              if (d.d.error) {
                this.responseError = d.d.error;
              } else {
                this.dialogRef.close(d.d.data);
              }
            },
            error: () => this.responseError = 'Something went wrong.',
            complete: () => this.waitForResponse = false,
          });
        this.client.createGroupChatter(this.data.group.id, response.user.id, response.user.login);
      });
  }
}
